// Copyright 2007 Edd Dawson.
// Distributed under the Boost Software License, Version 1.0.
// See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

// slam launcher executable for POSIX systems

// Compilation with GCC:
// g++ slam_posix.cpp -o slam -W -Wall -ansi -pedantic -O2

#include <string>
#include <list>
#include <vector>
#include <iostream>
#include <cstdlib>

#include <sys/types.h>
#include <unistd.h>

typedef std::vector<char> mystr_t;
typedef std::list<mystr_t> arglist_t;

char *make_arg(arglist_t &args, const std::string &s)
{
	mystr_t arg(s.begin(), s.end());
	arg.push_back(0);
	args.push_back(arg);
	return &*(args.rbegin()->begin());
}

int main(int argc, char **argv)
{
	const char *slam_root = std::getenv("SLAM_ROOT");
	if (!slam_root)
	{
		std::cerr << 	"SLAM_ROOT environment variable not set.\n"
						"Please set it to the absolute path of the\n"
						"directory that *contains* the 'slamcore'\n"
						"sub-directory and try again.\n";
		return 1;
	}

	// We need a char** to pass to execvp. The method is somewhat convoluted for the
	// sake of const-correctness.
	arglist_t args;
	std::vector<char *> new_argv;

	new_argv.push_back(make_arg(args, "python"));
	new_argv.push_back(make_arg(args, std::string(slam_root) + "/slam.py"));

	for (const char * const *arg = argv + 1; arg != argv + argc; ++arg)
		new_argv.push_back(make_arg(args, *arg));

	new_argv.push_back(static_cast<char *>(0)); // execvp must have it's argv NULL terminated

	char **base =  &*new_argv.begin();

	// execvp to launch slam's python script
	if (execvp(base[0], base))
	{
		std::cerr << "Failed to execute slam.py\n";
		return 3;
	}

	return 0;
}
