// Copyright 2007 Edd Dawson.
// Distributed under the Boost Software License, Version 1.0.
// See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

// slam launcher executable for Windows systems

// Compilation with MinGW:
// g++ slam_win32api.cpp -o slam -W -Wall -ansi -pedantic -O2

// Compilation with MSVC8:
// cl /nologo /D _CRT_SECURE_NO_DEPRECATE /D _SECURE_SCL=0 /D _HAS_ITERATOR_DEBUGGING=0 /O2 /Ox /Oy /W3 /EHsc /Zc:forScope,wchar_t slam_win32api.cpp /Feslam.exe

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <string>
#include <vector>
#include <iostream>

int main(int argc, char **argv)
{
	const char *slam_root = std::getenv("SLAM_ROOT");
	if (!slam_root)
	{
		std::cerr << 	"SLAM_ROOT environment variable not set.\n"
						"Please set it to the absolute path of the\n"
						"directory that *contains* the 'slamcore'\n"
						"sub-directory and try again.\n";
		return 1;
	}

	std::string cmd = "python \"";
	cmd += slam_root;
	cmd += "/slam.py\"";

	for (const char * const *arg = argv + 1; arg != argv + argc; ++arg)
	{
		cmd += " \"";
		cmd += *arg;
		cmd += "\"";
	}
	
	std::vector<char> cmdbuf(cmd.c_str(), cmd.c_str() + cmd.length() + 1);
	char * cmdraw = &*cmdbuf.begin(); // modifiable buffer
	
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );

	if (CreateProcess(NULL, cmdraw, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi))
	{
		WaitForSingleObject(pi.hProcess, INFINITE);
		CloseHandle(pi.hProcess);
    	CloseHandle(pi.hThread);
	}
	else
	{
		std::cerr << "Failed to start command: " << cmd << '\n';
		std::cerr << GetLastError() << '\n';
		return 1;
	}

	return 0;
}
