# Copyright 2007 Edd Dawson.
# Distributed under the Boost Software License, Version 1.0.
# See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

"""
This module contains slam messaging system, which is used to communicate information to the user.
"""

import threading
import sys
from sets import Set


verbose_ = True
warnings_mtx_ = threading.Lock()
warnings_ = Set() # we don't display the same warning more than once


def set_verbose(vb):
	"""
	Calling set_verbose(False) will mean that any messages given to info() will not be shown.
	"""
	global verbose_
	verbose_ = vb

def say(prefix, msg, details, stream=sys.stdout):
	"""
	Prints msg and the list of associated details to stream. Each detail is placed on its own line
	with '->' prepended. msg is printed with the given prefix and a colon at it's front.
	"""
	prevout = sys.stdout
	sys.stdout = stream

	try:
		print prefix + ': ' + msg
		for d in details:
			print '->', d
	except e:
		sys.stdout = prevout
		raise e

	sys.stdout = prevout


def warn(msg, details=[]):
	"""
	Prints a warning message to the error stream with the given details. Essentially, it just
	calls say('slam warning', msg, details, sys.stderr)
	"""
	warnings_mtx_.acquire()
	#if len(details) != 0: msg += reduce(lambda x,y: x+y, details)
	if not msg in warnings_:
		say('slam warning', msg, details, sys.stderr)
		warnings_.add(msg)
	warnings_mtx_.release()

def info(msg, details=[]):
	"""
	Prints an information message to the output stream with the given details. Essentially, it just
	calls say('slam', msg, details), unless set_verbose(False) has been called recently.
	"""
	global verbose_
	if verbose_:
		say('slam', msg, details)