# Copyright 2007 Edd Dawson.
# Distributed under the Boost Software License, Version 1.0.
# See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

"""
This module contains all functions relating to slam's minimal help system
"""

from slamcore.engine import message

slamfile_text_ = \
"""
#------------------------------------
# Generated by slam. Edit as needed.
#------------------------------------

# Compiles all source files with the
# 'usual' C++ filename extensions.

sources : common : *.cpp *.cxx *.c *.C *.cc *.CC

defines : debug : DEBUG
defines : release : NDEBUG

flags : common : strict iso
flags : debug : debug optimize-off
flags : release : optimize


# Object files are linked to create an
# executable called app or app.exe on
# Windows.

target : common : app
type : common : executable


# The directory for the object files
# and executable depend on the toolset
# name and the build variant.

dir : common : obj/$(toolset)/$(variant)

"""

def slam_version():
	"""
	Returns the current version of slam.
	This is displayed on every invocation of slam.
	"""
	return "0.2.10"

def dump_default_slam_file():
	"""
	Creates a default slamfile in the current working directory that globs up all source
	files with the usual C++ filename extensions and describes a build that compiles and
	links them in to an executable.
	"""
	f = open('slamfile', 'w')
	f.write(slamfile_text_)
	f.close()

def show_help():
	"""
	Prints the help message (-h on the command line) to the standard output stream.
	"""
	message.info('running version ' + slam_version())

	print '    -h    show this help message'
	print '    -ms   make a slamfile that compiles and links all C++ source'
	print '          files from this directory in to an executable'
	print '    -q    quiet mode; slam messages are supressed'
	print '    -i    show information on the selected toolset'
	print '    -nd   use the native dependency analysis facilities of the'
	print '          toolset, if available'
	print '    -r    recursively search the directory tree for slamfiles'
	print '    -w    wipe the cached dependency information for the current'
	print '          build variant and toolset combination'
	print '    -dr   dry run; show what would be built, any dependency'
	print '          updates are not written to the slam.db cache'
	print '    -c    clean; remove all build products'
	print '    -t    tidy; remove all intermediate build products'
	print "    -ne   do not copy the OS environment in to slam's"

	print '\nplease visit http://www.mr-edd.co.uk for a tutorial.'
