# Copyright 2007 Edd Dawson.
# Distributed under the Boost Software License, Version 1.0.
# See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

import slamcore.toolsets.gcc as gcc

class mingw(gcc.gcc):
	def __init__(self, env):
		gcc.gcc.__init__(self, env, 'lib', '.a', 'lib', '.dll', '', '.exe', '', '.o')

		self.map_flag(flag = 'threads', compiler = '-mthreads', linker = '-mthreads')
		self.map_flag(flag = 'console', linker = '-mconsole')
		self.map_flag(flag = 'gui', linker = '-mwindows')
		
		self.unmap_flag(flag = 'debug', compiler = '-ggdb3')
		self.map_flag(flag = 'debug', compiler = '-gstabs3')

		self.set_identifier('mingw', 2)

		desc = \
		"""
		This toolset uses MinGW's GCC-based g++ compiler to build C++ libraries
		and programs targeted for use on computers running Microsoft Windows.
		
		This toolset is provided with the slam distribution.
		"""
		
		issues = \
		"""
		I would also like to investigate the support of cross-compilation with
		this toolset.
		
		Support for resource files (.rc) is also desirable.
		"""

		features = \
		"""
		The MinGW toolset allows you to add the -mconsole and -mwindows switches
		to the linker by adding console and gui as arguments to the flags 
		directive, respectively. From the MinGW homepage:

		"The -mwindows switch is needed to create Windows executables instead of
		console applications. It assures the appropriate Windows libraries are
		linked in for you. To get a console screen along with a standard windows
		application, add the -mconsole flag as well as -mwindows."
		
		When I use MinGW, I like to add the console flag in debug builds of GUI
		applications, but not in relese builds:
		
		flags : debug : console # line in slamfile
		"""

		self.replace_note('Toolset description', desc)
		self.add_note('More information', 'The MinGW homepage: http://www.mingw.org')
		self.add_note('Known issues', issues)
		self.add_note('Special features', features)

def create(env):
	return mingw(env)