# Copyright 2007 Edd Dawson.
# Distributed under the Boost Software License, Version 1.0.
# See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

import slamcore.toolsets.msvcdotnet as msvcdotnet
import slamcore.toolsets.toolset as toolset


has_some = toolset.has_some
has_all = toolset.has_all
has_not = toolset.has_not
always = toolset.always

class msvc8(msvcdotnet.msvcdotnet):
    def __init__(self, env):
        msvcdotnet.msvcdotnet.__init__(self, env)

        # TODO: /bigobj? research this switch -- what are the negatives?
        self.map_flag(always(), compiler = '/wd4996')

        self.map_flag(has_some('iso', 'ansi'), compiler = '/Zc:forScope,wchar_t')

        self.map_flag(	has_some('optimize-min', 'optimize', 'optimize-max', 'optimize-size'), \
                        compiler = [('/D', '_SECURE_SCL=0'), ('/D', '_HAS_ITERATOR_DEBUGGING=0')] )

        self.unmap_flag(flag = 'optimize-floating-point', compiler = '/Op-')
        self.map_flag(flag = 'optimize-floating-point', compiler = '/fp:fast')

        self.set_identifier('msvc8', 3)
        
        desc = \
        """
        This toolset uses the 2005 edition of Microsoft's Visual C++ compiler 
        to build C++ libraries and programs.

        When using this toolset, you should make sure that the host environment
        is set up correctly, usually done by running the "Visual Studio Command
        Prompt", found somewhere in the Start menu.
        
        Standard C++ exception handling and run-time type identification (RTTI)
        are turned on by default with this toolset and those derived from it.

        Similarly, the _SCL_SECURE_NO_DEPRECATE preprocessor macro is defined on
        all builds because I believe it's silly not to have it (you shouldn't
        let kids play with knives). Warning 4996 is disabled by default, too.
        Of course whether you agree with this decision is a matter of taste.

        To change the default behaviours, you should derive your own toolset and
        customise to suit. This is much easier than it sounds!
        
        If you are using the Express edition of Visual C++ 2005, it is a good
        idea to install the MS Platform SDK (whether you use slam or not).
        
        Information on the using the Platform SDK with Visual C++ Express 2005
        can be found at:

        http://msdn.microsoft.com/platformsdk
        http://msdn.microsoft.com/vstudio/express/visualc/usingpsdk/

        This toolset is provided with the slam distribution.
        """

        self.replace_note('Toolset description', desc)

def create(env):
    return msvc8(env)