# Copyright 2007 Edd Dawson.
# Distributed under the Boost Software License, Version 1.0.
# See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

import os

from slamcore.engine.message import warn

msvc_switches_ = ['/?', '/AI', '/C', '/D', '/E', '/EHa', '/EHc', '/EHs', '/EP', '/F', '/FA', '/FC', '/FI', '/FR', '/FU', '/Fa', '/Fd', '/Fe', '/Fm', '/Fo', '/Fp', '/Fr', '/Fx', '/G3', '/G4', '/G5', '/G6', '/G7', '/GA', '/GB', '/GD', '/GF', '/GH', '/GL', '/GR', '/GS', '/GT', '/GX', '/GZ', '/Gd', '/Ge', '/Gf', '/Gh', '/Gm', '/Gr', '/Gs', '/Gy', '/Gz', '/H', '/I', '/J', '/LD', '/LDd', '/LN', '/MD', '/MDd', '/ML', '/MLd', '/MT', '/MTd', '/O1', '/O2', '/Oa', '/Ob', '/Od', '/Og', '/Oi', '/Op', '/Os', '/Ot', '/Ow', '/Ox', '/Oy', '/P', '/QI0f', '/QIfdiv', '/QIfist', '/RTC1', '/RTCc', '/RTCs', '/RTCu', '/TC', '/TP', '/Tc', '/Tp', '/U', '/V', '/W', '/WL', '/WX', '/Wall', '/Wp64', '/X', '/Y', '/Y-', '/YX', '/Yc', '/Yd', '/Yl', '/Yu', '/Z7', '/ZI', '/Za', '/Zc:', '/Zd', '/Ze', '/Zg', '/Zi', '/Zl', '/Zm', '/Zp', '/Zs', '/arch:', '/bigobj', '/c', '/clr', '/doc', '/errorReport', '/fp:', '/help', '/hotpatch', '/link', '/noBool', '/nologo', '/nox87copy', '/openmp', '/showIncludes', '/u', '/vd', '/vm', '/w', '/wd', '/we', '/wo', '@']


def is_msvc_switch(s):
	sw = s
	if sw[0] == '-': sw = '/' + sw[1:]

	for switch in msvc_switches_:
		if switch.startswith(sw): return True

	warn("removing unsupported compiler flag '" + s + "' wrongly suggested by pkg-config")
	return False

def split_pipe(cmd):
	return os.popen(cmd).read().strip().split()
	
# returns (defines, includes, libdirs, libs, compilerflags, linkerflags)
def parse(packages, msvc):
	if len(packages) == 0: return ([], [], [], [], [], [])

	# after creates lambdas that returns all characters in a string after the nth position
	after = lambda n: lambda s: s[n:]
	after2 = after(2)

	switch = ''
	if msvc:
		switch = '--msvc-syntax '
		format_lib = lambda x: x[:-4]
		format_dir = after(9)
		is_define = lambda x: x.startswith('/D') or x.startswith('-D')
		is_include = lambda x: x.startswith('/I') or x.startswith('-I')
	else:
		format_lib = after2
		format_dir = after2
		is_define = lambda x: x.startswith('-D')
		is_include = lambda x: x.startswith('-I')
		
	format_define = after2
	format_include = after2

	packages = ' '.join(packages)

	libs = [format_lib(l) for l in split_pipe('pkg-config --libs-only-l ' + switch + packages)]
	libdirs = [format_dir(d) for d in split_pipe('pkg-config --libs-only-L ' + switch + packages)]
	linkerflags = split_pipe('pkg-config --libs-only-other ' + switch + packages)
	defines = []
	includes = []
	compilerflags = []

	options = split_pipe('pkg-config --cflags ' + switch + packages)
	for opt in options:
		if is_define(opt) : defines.append(format_define(opt))
		elif is_include(opt) : includes.append(format_include(opt))
		else: compilerflags.append(opt)
		
	if msvc:
		compilerflags = [c for c in compilerflags if is_msvc_switch(c)]
		
	return (defines, includes, libdirs, libs, compilerflags, linkerflags)