# Copyright 2007 Edd Dawson.
# Distributed under the Boost Software License, Version 1.0.
# See accompanying file BOOST_LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt

"""
This module provides helper functions for slam's toolsets
"""

import os
import threading
from slamcore.engine import message
from slamcore.engine.exceptions import slam_cli_error

dir_mutex_ = threading.Lock()

def get_cmd_output(cmd):
	"""
	Returns the output of a system command as a string. If the command failed, a
	slam_cli_error exception is raised
	"""
	child = os.popen(cmd)
	data = child.read()
	err = child.close()
	if err:
		raise slam_cli_error('%s failed with exit code %d' % (cmd, err))

	return data

def execute_cmd(cmd, expected = 0):
	"""
	Runs the system command given by cmd, showing its output to the console. If the command returns
	something other than expected, a slam_cli_error exception is raised.
	"""
	print cmd
	code = os.system(cmd)
	if code != expected:
		raise slam_cli_error('system call returned unexpected exit-code %d' % (code))

def make_last_path_segment(path, mode):
	"""
	Used to implement ensure_dir_exists. Creates the directory given by path and if it doesn't 
	already exist, places a file names '.slam_made_this' inside. Creates any intermediate
	directories with a recusrive call. mode should be an integer that specifies the permissions
	of the directories generated
	"""
	if os.path.exists(path):
		if os.path.isfile(path):
			raise slam_cli_error('encountered file %s when trying to create directory' % (path))
		else:
			return

	# The path doesn't exist. We need to make sure its parent directory does
	# before making it
	popped_path = os.path.split(path)[0]
	if popped_path and len(popped_path) > 0:
		make_last_path_segment(popped_path, mode)

	# Indicate that slam made this directory so we can remove it on -c or -t
	os.mkdir(path, mode)
	open(os.path.join(path, '.slam_made_this'), 'wb').close()


def ensure_dir_exists(path, mode = 0755):
	"""
	Creates the directory given by path and any intermediate paths that don't exist. Places a file 
	named '.slam_made_this' inside each directory created. mode should be an integer that specifies
	the permissions	of the directories generated.
	"""
	dir_mutex_.acquire()
	try:
		if not os.path.exists(path):
			message.info('making directory ' + path)
			make_last_path_segment(path, mode)

	except RuntimeError, err:
		dir_mutex_.release()
		raise err
	except:
		pass # directory already exists (we hope -- a bit dodgy!)

	dir_mutex_.release()

def remove_file(filename):
	"""
	Removes the file with the given name if it exists, otherwise does nothing.
	"""
	if os.path.exists(filename) and os.path.isfile(filename):
		message.info('removing ' + filename)
		os.remove(filename)
